@@ -14,19 +14,25 @@ class JbuilderGeneratorTest < Rails::Generators::TestCase
     %w(index show).each do |view|
       assert_file "app/views/posts/#{view}.json.jbuilder"
     end
+    assert_file "app/views/posts/_post.json.jbuilder"
   end
 
   test 'index content' do
     run_generator
 
     assert_file 'app/views/posts/index.json.jbuilder' do |content|
-      assert_match /json\.array!\(@posts\) do \|post\|/, content
-      assert_match /json\.extract! post, :id, :title, :body/, content
-      assert_match /json\.url post_url\(post, format: :json\)/, content
+      assert_match %r{json.array! @posts, partial: 'posts/post', as: :post}, content
     end
 
     assert_file 'app/views/posts/show.json.jbuilder' do |content|
-      assert_match /json\.extract! @post, :id, :title, :body, :created_at, :updated_at/, content
+      assert_match %r{json.partial! \"posts/post\", post: @post}, content
     end
+    
+    assert_file 'app/views/posts/_post.json.jbuilder' do |content|            
+      assert_match %r{json\.extract! post, :id, :title, :body}, content
+      assert_match %r{json\.url post_url\(post, format: :json\)}, content
+    end
+    
+
   end
 end
