@@ -62,9 +62,9 @@ class JbuilderTemplateTest < ActionView::TestCase
     Rails.cache.clear
   end
 
-  def jbuild(source)
+  def jbuild(source, options = {})
     @rendered = []
-    partials = PARTIALS.clone
+    partials = options.fetch(:partials, PARTIALS).clone
     partials["test.json.jbuilder"] = source
     resolver = ActionView::FixtureResolver.new(partials)
     lookup_context.view_paths = [resolver]
@@ -307,18 +307,6 @@ class JbuilderTemplateTest < ActionView::TestCase
     assert_equal %w[a b c], result
   end
 
-  test "fragment caching works with previous version of cache digests" do
-    undef_context_methods :cache_fragment_name
-
-    @context.expects :fragment_name_with_digest
-
-    jbuild <<-JBUILDER
-      json.cache! "cachekey" do
-        json.name "Cache"
-      end
-    JBUILDER
-  end
-
   test "fragment caching works with current cache digests" do
     undef_context_methods :fragment_name_with_digest
 
@@ -332,6 +320,35 @@ class JbuilderTemplateTest < ActionView::TestCase
     JBUILDER
   end
 
+  test "fragment caching uses fragment_cache_key" do
+    undef_context_methods :fragment_name_with_digest, :cache_fragment_name
+
+    @context.expects(:fragment_cache_key).with("cachekey")
+
+    jbuild <<-JBUILDER
+      json.cache! "cachekey" do
+        json.name "Cache"
+      end
+    JBUILDER
+  end
+
+  test "fragment caching instrumentation" do
+    undef_context_methods :fragment_name_with_digest, :cache_fragment_name
+
+    payloads = {}
+    ActiveSupport::Notifications.subscribe("read_fragment.action_controller") { |*args| payloads[:read_fragment] = args.last }
+    ActiveSupport::Notifications.subscribe("write_fragment.action_controller") { |*args| payloads[:write_fragment] = args.last }
+
+    jbuild <<-JBUILDER
+      json.cache! "cachekey" do
+        json.name "Cache"
+      end
+    JBUILDER
+
+    assert_equal "jbuilder/cachekey", payloads[:read_fragment][:key]
+    assert_equal "jbuilder/cachekey", payloads[:write_fragment][:key]
+  end
+
   test "current cache digest option accepts options" do
     undef_context_methods :fragment_name_with_digest
 
@@ -345,6 +362,47 @@ class JbuilderTemplateTest < ActionView::TestCase
     JBUILDER
   end
 
+  test "fragment caching accepts expires_in option" do
+    undef_context_methods :fragment_name_with_digest
+
+    @context.expects(:cache_fragment_name).with("cachekey", {})
+
+    jbuild <<-JBUILDER
+      json.cache! "cachekey", expires_in: 1.minute do
+        json.name "Cache"
+      end
+    JBUILDER
+  end
+
+  test "caching root structure" do
+    undef_context_methods :fragment_name_with_digest, :cache_fragment_name
+
+    cache_miss_result = jbuild <<-JBUILDER
+      json.cache_root! "cachekey" do
+        json.name "Miss"
+      end
+    JBUILDER
+
+    cache_hit_result = jbuild <<-JBUILDER
+      json.cache_root! "cachekey" do
+        json.name "Hit"
+      end
+    JBUILDER
+
+    assert_equal cache_miss_result, cache_hit_result
+  end
+
+  test "failing to cache root after attributes have been defined" do
+    assert_raises ActionView::Template::Error, "cache_root! can't be used after JSON structures have been defined" do
+      jbuild <<-JBUILDER
+        json.name "Kaboom"
+        json.cache_root! "cachekey" do
+          json.name "Miss"
+        end
+      JBUILDER
+    end
+  end
+
   test "does not perform caching when controller.perform_caching is false" do
     controller.perform_caching = false
 
@@ -380,4 +438,20 @@ class JbuilderTemplateTest < ActionView::TestCase
     assert_equal 123, result["id"]
     assert_equal "Chris Harris", result["name"]
   end
+
+  test "renders partial via set! with same name as HTML partial" do
+    partials = {
+      "_blog_post.html.erb" => "Hello!",
+      "_blog_post.json.jbuilder" => BLOG_POST_PARTIAL
+    }
+
+    @post = BLOG_POST_COLLECTION.first
+
+    result = jbuild(<<-JBUILDER, partials: partials)
+      json.post @post, partial: "blog_post", as: :blog_post
+    JBUILDER
+
+    assert_not_nil result["post"]
+    assert_equal 1, result["post"]["id"]
+  end
 end
